#'
#' Get details about the profiles available to the logged-in user.
#'
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @return Profiles available to the logged-in user.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' profiles <- h2osteam.get_profiles(conn)
#' print(profiles)
#' }
#' 
#' @export
#' 
h2osteam.get_profiles <- function(conn) {
  profiles <- .getProfiles(conn)
  profiles
}

#'
#' Import H2O engine from Enterprise Steam server and makes it available to users.
#' 
#' Only Enterprise Steam administrators are authorized to call this method.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param path Full path to the H2O engine on disk of the Enterprise Steam server.
#' @return None
#' 
#' @examples 
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.import_h2o_engine(conn, "/tmp/h2o-3.26.0.6-cdh6.3.zip")
#' }
#' 
#' @export
#' 
h2osteam.import_h2o_engine <- function(conn, path) {
  void <- .importH2oEngine(conn, path)
}

#'
#' Lists H2O engines from Enterprise Steam server.
#'
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @return list of available H2O engines
#'
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.get_h2o_engines(conn)
#' }
#'
#' @export
#'
h2osteam.get_h2o_engines <- function(conn) {
  engines <- .getH2oEngines(conn)
  engines
}

#'
#' Lists H2O Kubernetes engines from Enterprise Steam server.
#'
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @return list of available H2O Kubernetes engines
#'
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.get_h2o_kubernetes_engines(conn)
#' }
#'
#' @export
#'
h2osteam.get_h2o_kubernetes_engines <- function(conn) {
  engines <- .getH2oKubernetesEngines(conn)
  engines
}
