#' 
#' Launch a new H2O cluster on Hadoop.
#'
#' Launches a new H2O cluster on Hadoop using the parameters described below.
#' You do not need to specify all parameters. In that case they will be filled
#' based on the default value of the selected profile.
#' The process of launching a cluster can take up to 5 minutes.
#'
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name Name of the cluster.
#' @param version Version of H2O that will be used in the cluster.
#' @param profile_name (Optional) Specify name of an existing profile that will be used for this cluster.
#' @param nodes (Optional) Number of nodes of the H2O cluster.
#' @param node_cpus (Optional) Number of CPUs/threads used by H2O on a single node. Specify '0' to use all available CPUs/threads.
#' @param yarn_vcores (Optional) Number of YARN virtual cores per cluster node. Should match node_cpus.
#' @param node_memory_gb (Optional) Amount of memory in GB allocated for a single H2O node.
#' @param extra_memory_percent (Optional) Percentage of extra memory that will be allocated outside of H2O JVM for algos like XGBoost.
#' @param max_idle_h (Optional) Maximum amount of time in hours the cluster can be idle before shutting down.
#' @param max_uptime_h (Optional) Maximum amount of time in hours the cluster will be up before shutting down.
#' @param timeout_s (Optional) Maximum amount of time in seconds to wait for the H2O cluster to start.
#' @param yarn_queue (Optional) Name of the YARN queue where the cluster will be placed.
#' @param leader_node_id (Optional) ID of the H2O leader node.
#' @return H2O cluster connection configuration that can be passed to \code{h2o.connect(config = config)}.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' config <- h2osteam.launch_h2o_cluster(conn, name = "test-cluster", version = "3.30.0.1",
#'                                       nodes = 5, node_memory_gb = 20)
#' h2o.connect(config = config)
#' }
#'
#' @export
#' 
h2osteam.launch_h2o_cluster <- function(conn,
                               name = NA_character_,
                               version = NA_character_,
                               profile_name = NA_character_,
                               nodes = NA_integer_,
                               node_cpus = NA_integer_,
                               yarn_vcores = NA_integer_,
                               node_memory_gb = NA_integer_,
                               extra_memory_percent = NA_integer_,
                               max_idle_h = NA_integer_,
                               max_uptime_h = NA_integer_,
                               timeout_s = NA_integer_,
                               yarn_queue = "",
                               leader_node_id = 0) {
  
    # Validate required parameters
    if (is.na(name)) {
      stop("Must enter valid cluster name")
    }
    if (is.na(version)) {
      stop("Must enter H2O version")
    }
    if (is.na(profile_name)) {
      profile_name <- "default-h2o"
    }
    
    # Load selected profile
    selected_profile <- .getProfileByName(conn, profile_name)$profile
    profile <- selected_profile$h2o
   
    # Replace unset paramaters by default profile parameters
    if (is.na(nodes)) {
      nodes <- profile$h2o_nodes$initial
    }
    if (is.na(node_cpus)) {
      node_cpus <- profile$h2o_threads$initial
    }
    if (is.na(yarn_vcores)) {
      yarn_vcores <- profile$yarn_vcores$initial
    }
    if (is.na(node_memory_gb)) {
      node_memory_gb <- profile$h2o_memory$initial
    }
    if (is.na(extra_memory_percent)) {
      extra_memory_percent <- profile$h2o_extramempercent$initial
    }
    if (is.na(max_idle_h)) {
      max_idle_h <- profile$max_idle_time$initial
    }
    if (is.na(max_uptime_h)) {
      max_uptime_h <- profile$max_uptime$initial
    }
    if (is.na(timeout_s)) {
      timeout_s <- profile$start_timeout$initial
    }

    # Get H2O engine ID
    h2o_engine_id <- .getH2oEngineByVersion(conn, version)$engine$id
    
    # Launch H2O cluster
    id <- .launchH2oCluster(conn=conn, parameters=list(
      name = name,
      h2o_engine_id = h2o_engine_id,
      profile_id = selected_profile$id,

      h2o_nodes = nodes,
      h2o_threads = node_cpus,
      yarn_vcores = yarn_vcores,
      h2o_memory = node_memory_gb,
      h2o_extramempercent = extra_memory_percent,

      max_idle_time = max_idle_h,
      max_uptime = max_uptime_h,
      start_timeout = timeout_s,

      yarn_queue = yarn_queue,
      leader_node_id = leader_node_id
    ))$id
    
    print("H2O Cluster is starting, please wait...")
    
    # Wait until H2O cluster starts or fails
    .h2o_cluster_wait(conn, id)
    
    # get H2O cluster details
    cluster <- .getH2oCluster(conn, id)$cluster
    
    # Get config for h2o.connect()
    .get_h2o_cluster_config(conn, cluster)
}

#'
#' Get connection configuration of H2O cluster on Hadoop.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name The name of the H2O cluster.
#' @return H2O cluster connection configuration that can be passed to \code{h2o.connect(config = config)}.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' config <- h2osteam.get_h2o_cluster(conn, "my-cluster")
#' h2o.connect(config = config)
#' }
#'
#' @export
#' 
h2osteam.get_h2o_cluster <- function(conn, name) {
    cluster <- .getH2oClusterByName(conn, name)$cluster
    .get_h2o_cluster_config(conn, cluster)
}

#'
#' Get information about all clusters from the Enterprise Steam interface
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @return The list of H2O clusters.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' clusters <- h2osteam.get_h2o_clusters(conn)
#' print(clusters)
#' }
#'
#' @export
#' 
h2osteam.get_h2o_clusters <- function(conn) {
  .getH2oClusters(conn)$clusters
}

#'
#' Stop a running H2O cluster on Hadoop.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name The name of the H2O cluster.
#' @return None
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.stop_h2o_cluster(conn, "my-cluster")
#' }
#'
#' @export
#' 
h2osteam.stop_h2o_cluster <- function(conn, name) {
    cluster <- .getH2oClusterByName(conn, name)$cluster
    print("H2O Cluster is stopping, please wait...")
    .stopH2oCluster(conn, cluster$id, FALSE)
    # Wait until H2O cluster stops or fails
    ok <- .h2o_cluster_stop_wait(conn, cluster$id)
}

#'
#' Delete (Terminate) a stopped H2O cluster on Hadoop.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name The name of the H2O cluster.
#' @return None
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.delete_h2o_cluster(conn, "my-cluster")
#' }
#'
#' @export
#' 
h2osteam.delete_h2o_cluster <- function(conn, name) {
    cluster <- .getH2oClusterByName(conn, name)$cluster
    ok <- .deleteH2oCluster(conn, cluster$id)
}

#'
#' @keywords internal
#' @noRd
#'
.h2o_cluster_wait <- function(conn, cluster_id) {
  current_status <- .get_h2o_cluster_status(conn, cluster_id)
  
  while(TRUE) {
      new_status <- .get_h2o_cluster_status(conn, cluster_id)
      
      if (current_status != new_status) {
          current_status <- new_status
      }
      
      if (current_status == "starting" || current_status == "connecting") {
          Sys.sleep(5)
          next
      } else if (current_status == "running") {
          print("H2O cluster is running")
          break
      } else {
          print("H2O cluster failed to start")
          break
      }
  }
}

#'
#' @keywords internal
#' @noRd
#'
.h2o_cluster_stop_wait <- function(conn, cluster_id) {
  current_status <- .get_h2o_cluster_status(conn, cluster_id)

  while(TRUE) {
    new_status <- .get_h2o_cluster_status(conn, cluster_id)

    if (current_status != new_status) {
      current_status <- new_status
    }

    if (current_status == "stopping") {
      Sys.sleep(5)
      next
    } else if (current_status == "stopped") {
      print("H2O cluster is stopped")
      break
    } else {
      print("H2O cluster failed to stop")
      break
    }
  }
}

#'
#' @keywords internal
#' @noRd
#'
.get_h2o_cluster_status <- function(conn, cluster_id) {
  .getH2oCluster(conn, cluster_id)$cluster$status
}

#'
#' @keywords internal
#' @noRd
#'
.get_h2o_cluster_config <- function(conn, cluster) {
    list(
      connect_params = list(
        https = TRUE,
        insecure = !conn$verify_ssl,
        context_path = sub("/", "", cluster$context_path),
        cookies = conn$cookie,
        ip = conn$host,
        port = strtoi(conn$port)
      )
    )
}