#' 
#' Launch a new H2O cluster on Kubernetes.
#'
#' Launches a new H2O cluster on Kubernetes using the parameters described below.
#' You do not need to specify all parameters. In that case they will be filled
#' based on the default value of the selected profile.
#' The process of launching a cluster can take up to 5 minutes.
#'
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name Name of the cluster.
#' @param profile_name (Optional) Specify name of an existing profile that will be used for this cluster.
#' @param version Version of H2O that will be used in the cluster.
#' @param dataset_size_gb (Optional) Specify size of your uncompressed dataset. For compressed data source, use dataset_dimension parameter. Cluster parameters will be preset to accommodate your dataset within selected profile limits. Does not override user-specified values. 
#' @param dataset_dimension (Optional) Array of (n_rows, n_cols) representing an estimation of dataset dimensions. Use this parameter when you intend to use compressed data source like Parquet format. Cluster parameters will be preset to accommodate your dataset within selected profile limits. Does not override user-specified values.
#' @param nodes (Optional) Number of nodes of the H2O cluster.
#' @param node_cpus (Optional) Number of CPUs/threads used by H2O on a single node. Specify '0' to use all available CPUs/threads.
#' @param node_gpus (Optional) Number of GPUs per node
#' @param node_memory_gb (Optional) Amount of memory in GB allocated for a single H2O node.
#' @param max_idle_h (Optional) Maximum amount of time in hours the cluster can be idle before shutting down.
#' @param max_uptime_h (Optional) Maximum amount of time in hours the cluster will be up before shutting down.
#' @param timeout_s (Optional) Maximum amount of time in seconds to wait for the H2O cluster to start.
#' @param volumes (Optional) Specify unbound volumes to mount with this instance.
#' @return H2O cluster connection configuration that can be passed to \code{h2o.connect(config = config)}.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' config <- h2osteam.launch_h2o_kubernetes_cluster(conn, name = "test-cluster", version = "3.30.0.1",
#'                                       nodes = 5, node_memory_gb = 20)
#' h2o.connect(config = config)
#' }
#'
#' @export
#'
h2osteam.launch_h2o_kubernetes_cluster <- function(conn,
                               name = NA_character_,
                               profile_name = NA_character_,
                               version = NA_character_,
                               dataset_size_gb = 0,
			       dataset_dimension = c(0, 0),
			       nodes = NA_integer_,
                               node_cpus = NA_integer_,
			       node_gpus = NA_integer_,
                               node_memory_gb = NA_integer_,
                               max_idle_h = NA_integer_,
                               max_uptime_h = NA_integer_,
                               timeout_s = NA_integer_,
			       volumes = NA_character_) {
  
    # Validate required parameters
    if (is.na(name)) {
      stop("Must enter valid cluster name")
    }
    if (is.na(version)) {
      stop("Must enter H2O version")
    }
    if (is.na(profile_name)) {
      profile_name <- "default-h2o-kubernetes"
    }

    profile <- .getProfileByName(conn, profile_name)$profile$h2o_kubernetes

    # Replace unset paramaters by default profile parameters
    if (is.na(nodes)) {
      nodes <- as.integer(profile$node_count$initial)
    }
    if (is.na(node_cpus)) {
      node_cpus <- as.integer(profile$cpu_count$initial)
    }
    if (is.na(node_memory_gb)) {
      node_memory_gb <- as.integer(profile$memory_gb$initial)
    } 
    if (is.na(max_idle_h)) {
      max_idle_h <- as.integer(profile$max_idle_hours$initial)
    }
    if (is.na(max_uptime_h)) {
      max_uptime_h <- as.integer(profile$max_uptime_hours$initial)
    }
    if (is.na(timeout_s)) {
      timeout_s <- as.integer(profile$timeout_seconds$initial)
    }

    mem <- .getEstimatedClusterMemory(conn=conn, parameters=list(
		dataset_size_gb = dataset_size_gb,
		rows = dataset_dimension[1],
		cols = dataset_dimension[2],
		using_x_g_boost = FALSE))$cluster_memory

    # Launch H2O cluster
    id <- .launchH2oKubernetesCluster(conn=conn, parameters=list(
      name = name,
      profile_name = profile_name,
      version = version,

      dataset_size_gb = dataset_size_gb,
      dataset_dimension = dataset_dimension,
      node_count = nodes,
      cpu_count = node_cpus,
      memory_gb = node_memory_gb,

      max_idle_hours = max_idle_h,
      max_uptime_hours = max_uptime_h,
      timeout_seconds = timeout_s,
      rec_memory = as.integer(mem$total_memory_gb),

      volumes = volumes
    ))$cluster$id
    
    print("H2O Cluster is starting, please wait...")

    # Wait until H2O cluster starts or fails
    .h2o_kubernetes_cluster_wait(conn, id)
    
    # get H2O cluster details
    cluster <- .getH2oKubernetesClusterById(conn, id)$cluster
    
    # Get config for h2o.connect()
    .get_h2o_kubernetes_cluster_config(conn, cluster)
}

#'
#' Get connection configuration of H2O cluster on Kubernetes.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name The name of the H2O kubernetes cluster.
#' @return H2O cluster connection configuration that can be passed to \code{h2o.connect(config = config)}.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' config <- h2osteam.get_h2o_kubernetes_cluster(conn, "my-cluster")
#' h2o.connect(config = config)
#' }
#'
#' @export
#' 
h2osteam.get_h2o_kubernetes_cluster <- function(conn, name) {
    cluster <- .getH2oKubernetesClusterByName(conn, name)$cluster
    .get_h2o_kubernetes_cluster_config(conn, cluster)
}

#'
#' Get information about all H2O clusters on Kubernetes
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @return The list of H2O clusters.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' clusters <- h2osteam.get_h2o_kubernetes_clusters(conn)
#' print(clusters)
#' }
#'
#' @export
#' 
h2osteam.get_h2o_kubernetes_clusters <- function(conn) {
  .getH2oKubernetesClusters(conn)$cluster
}

#'
#' Stop a running H2O cluster on Kubernetes.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name The name of the H2O kubernetes cluster.
#' @return None
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.stop_h2o_kubernetes_cluster(conn, "my-cluster")
#' }
#'
#' @export
#' 
h2osteam.stop_h2o_kubernetes_cluster <- function(conn, name) {
    cluster <- .getH2oKubernetesClusterByName(conn, name)$cluster
    print("H2O Cluster is stopping, please wait...")
    .stopH2oKubernetesClusterById(conn, cluster$id)
    # Wait until H2O cluster stops or fails
    ok <- .h2o_kubernetes_cluster_wait(conn, cluster$id)
}

#'
#' Delete (Terminate) a stopped H2O cluster on Kubernetes.
#' 
#' @param conn The \code{SteamConnection} object obtained by \code{h2osteam.login}.
#' @param name The name of the H2O kubernetes cluster.
#' @return None
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.h2o.ai:9555",
#'                        username = "user01", password = "token-here")
#' h2osteam.delete_h2o_kubernetes_cluster(conn, "my-cluster")
#' }
#'
#' @export
#' 
h2osteam.delete_h2o_kubernetes_cluster <- function(conn, name) {
    ok <- .deleteH2oKubernetesCluster(conn, name)
}

.h2o_kubernetes_cluster_wait <- function(conn, cluster_id) {
	cluster <- .getH2oKubernetesClusterById(conn, cluster_id)$cluster
	current_status <- cluster$status
	target <- cluster$target_status

	while(current_status != target) {
		Sys.sleep(5)
		cluster <- .getH2oKubernetesClusterById(conn, cluster_id)$cluster
		current_status <- cluster$status
		target <- cluster$target_status
	}
}

#'
#' @keywords internal
#' @noRd
#'
.get_h2o_kubernetes_cluster_status <- function(conn, cluster_id) {
  .getH2oKubernetesCluster(conn, cluster_id)$cluster$status
}

#'
#' @keywords internal
#' @noRd
#'
.get_h2o_kubernetes_cluster_config <- function(conn, cluster) {
    list(
      connect_params = list(
        https = TRUE,
        insecure = !conn$verify_ssl,
        context_path = sub("/", "", cluster$context_path),
        cookies = conn$cookie,
        ip = conn$host,
        port = strtoi(conn$port)
      )
    )
}
