#'
#' Connect and login to an existing Enterprise Server server.
#'
#' @param url Object of class \code{character} representing the Full URL (including schema and port) of the Enterprise Steam server.
#' @param username Object of class \code{character} representing the username.
#' @param password Object of class \code{character} representing user's password or access token.
#' @param verify_ssl (Optional) A \code{logical} value indicating whether to verify SSL when connecting to the server.
#' @param cacert (Optional) Path to a CA bundle file with root and intermediate certificates of trusted CAs.
#' @return This method will connect to the Enterprise Steam server and return a \code{SteamConnection} object that must be passed to subsequent methods.
#' 
#' @examples
#' \dontrun{
#' conn <- h2osteam.login(url = "https://steam.example.com:9555",
#'                        username = "AzureDiamond", password = "hunter2")
#' }
#' 
#' @importFrom httr POST config set_config authenticate stop_for_status handle content cookies
#' @importFrom utils packageVersion
#' @importFrom urltools url_parse url_compose
#' @export
#'
h2osteam.login <- function(url = NA_character_, username = NA_character_, password = NA_character_, verify_ssl = TRUE, cacert = "") {
  # Validate parameter
  if (is.na(url)) {
    stop("Parameter 'url' must be set")
  }
  
  if (is.na(username)) {
    stop("Parameter 'username' must be set")
  }
  
  if (is.na(password)) {
    stop("Parameter 'password' must be set")
  }
  
  # Parse URL
  url <- url_parse(url)
  
  # Clean URL
  url$scheme = "https"
  url$path = NA_character_
  url$parameter = NA_character_
  url$fragment = NA_character_
  if (is.na(url$port)) {
    url$port <- 443
  }
  
  # Save host and port before rebuilding URL
  host <- url$domain
  port <- url$port
  
  # Rebuild URL
  url <- url_compose(url)
  
  # Create handle
  h <- handle(url)
  
  # Verify SSL
  if (verify_ssl == FALSE) {
    set_config(config(ssl_verifypeer = 0L, ssl_verifyhost = 0L))
  } else {
    if (cacert != "") {
      set_config(config(ssl_verifypeer = 1L, ssl_verifyhost = 2L, cainfo = cacert))
    } else {
      set_config(config(ssl_verifypeer = 1L, ssl_verifyhost = 2L))
    }
  }
  
  # Login
  authURL = paste0(url, "auth")
  r = POST(url = authURL, handle = h, config = authenticate(username, password))
  stop_for_status(r, paste("login:", trimws(content(r))))
  
  # Extract cookies
  cookie = ""
  c = cookies(r)
  for (i in 1:nrow(c)) {
    if (c[i, "name"] == "steam-session") {
      cookie = paste0("steam-session=", c[i, "value"])
    }
  }
  
  # Create conn object
  conn <- structure(
    list(url=url, host=host, port=port, verify_ssl=verify_ssl, cacert=cacert, username=username, password=password, handle=h, cookie = cookie, uid=0),
    class="SteamConnection"
  )
  
  # Test connection
  client_api_version <- packageVersion("h2osteam")
  server_api_version <- .pingServer(conn, "R connect")
  if (server_api_version != client_api_version) {
    stop(sprintf("Client API version '%s' does not match server API version '%s'", client_api_version, server_api_version))
  }
  
  conn
}